# -*- coding: utf-8 -*-
import sys
from django.conf import settings

def autodiscover_handlers():
    """
    Register signal handlers from each addon.
    """
    for addon in settings.ADDONS:
        fullname = "%s.%s.handlers" % (settings.ADDONS_PREFIX, addon)
        try:
            __import__(fullname)
        except ImportError, err:
            if str(err) != "No module named handlers":
                raise err

        if fullname in sys.modules:
            handlers = sys.modules[fullname]
            handlers.connect()

def autodiscover_notifications():
    """
    Register notifications from each addon.
    TODO: Implement this with signals. Requires changes in django-notifications upstream.
    """
    for addon in settings.ADDONS:
        fullname = "%s.%s.notifications" % (settings.ADDONS_PREFIX, addon)
        try:
            __import__(fullname)
        except ImportError:
            pass

def autodiscover():
    autodiscover_notifications()
    autodiscover_handlers()



# -*- coding: utf-8 -*-

"""
This is django_addons.urls module, it loads URLconf of
each addon.

In main urls you should have something like:

  urlpatterns += patterns('', (r'', include('django_addons.urls')))

"""

import os
from django.conf.urls.defaults import *
from django.conf import settings
from views import addons_view

urlpatterns = patterns('')

if settings.DEBUG:
    urlpatterns += patterns('', url(r'^addons/$', addons_view))

for addon in settings.ADDONS:
    filename = "%s/%s/urls.py" % (settings.ADDONS_ROOT, addon)
    fullname = "%s.%s.urls" % (settings.ADDONS_PREFIX, addon)
    if os.path.exists(filename):
        urlpatterns += patterns('',
            url(r'', include(fullname)),
        )



# -*- coding: utf-8 -*-

import os
import sys

from django.conf import settings
from django.conf.urls.defaults import *

VERSION = (0, 4, 0, 'final')

def get_version():
    version = '%s.%s' % (VERSION[0], VERSION[1])
    if VERSION[2]:
        version = '%s.%s' % (version, VERSION[2])
    if VERSION[3] != 'final':
        version = '%s %s' % (version, VERSION[3])
    return version

def dependency_found(addon):
    return addon in ( \
        getattr(settings, "ADDONS", []) + 
        getattr(settings, "ADDONS_PROVIDED", []))

def get_meta(addon):
    # Load metainformation
    name = "%s.%s" % (settings.ADDONS_PREFIX, addon)
    try:
        mod = None
        __import__(name)
        mod = sys.modules[name]
    except Exception, e:
        print e
        pass
    finally:
        return getattr(mod, 'Meta', None)






# -*- coding: utf-8 -*-
import os
import sys
from django.core.management.base import BaseCommand
from optparse import make_option
from django.conf import settings
from django_addons import get_meta

class Command(BaseCommand):
    option_list = BaseCommand.option_list + (
        make_option('--enable', '-e', default=None, dest='addon_to_enable',
            help='Enable a addon'),
        make_option('--disable', '-d', default=None, dest='addon_to_disable',
            help='Disable a addon'),
    )
    help = "List, enable or disable addons."

    requires_model_validation = False
    can_import_settings = False

    def handle(self, *args, **options):
        def save():
            print "Saving to:", settings.ADDONS_DISABLED_CONF
            fh = open( settings.ADDONS_DISABLED_CONF, "w")
            fh.write("# This file is automatically generated by ./manage.py addons\n")
            fh.write("# To enable an addon use: ./manage.py addons -e ADDON_NAME\n")
            fh.write("# To disable an addon use: ./manage.py addons -d ADDON_NAME\n")
            fh.write("ADDONS_DISABLED = [ %s ]\n" % ", ".join("'%s'" % x for x in settings.ADDONS_DISABLED))
            fh.close()

        def addon_enable(addon):
            if not addon:
                return
            if addon in settings.ADDONS_DISABLED:
                print "Enabling addon: %s" % addon
                settings.ADDONS_DISABLED.remove(addon)
                save()
            if not addon in settings.ADDONS:
                settings.ADDONS.append(addon)

        def addon_disable(addon):
            if not addon:
                return
            if addon in settings.ADDONS:
                if addon not in settings.ADDONS_DISABLED:
                    print "Disabling addon: %s" % addon
                    settings.ADDONS.remove(addon)
                    settings.ADDONS_DISABLED.append(addon)
                    save()
                    return True

        # Enable addon if we have to
        addon_enable(options.get("addon_to_enable"))
        # Disable addon if we have to
        addon_disable(options.get("addon_to_disable"))

        # Otherwise display list of addon
        print "Available addons:"
        for addon in settings.ADDONS:


            print " * %s" % addon,
            meta = get_meta(addon)
            if meta:
                print "- %s" % meta.title,
            print

        for addon in settings.ADDONS_DISABLED:
            print " * %s (disabled)" % addon



# -*- coding: utf-8 -*-
import re
from django import template
from django.template import Node, Variable, TemplateSyntaxError, TemplateDoesNotExist
from django.template.loader import get_template_from_string, find_template_source
import settings
from django.template.loader_tags import BlockNode

"""
This module adds support for hook tag which allows doing magic like this:

In your Django project define a piece of page which imports some stuff from *all* addons:
/MyDjangoProject/templates/main.html

  {% load addons %}
  <ul>
  <li>Menuitem 1</li>
  <li>Menuitem 2</li>
  {% hook "main.html" navmenu %}
  </ul>

In a addon define: 
/MyDjangoProject/addons/MyAddon/templates/MyAddon/main.html:
  {% block navmenu %}
  <li>Custom menuitem</li>
  {% endblock }

In some other addon define:
/MyDjangoProject/addons/MyOtherAddon/templates/MyOtherAddon/main.html:
  {% block navmenu %}
  <li>Other menuitem</li>
  {% endblock }

This all ends up rendered as:

  <ul>
  <li>Menuitem 1</li>
  <li>Menuitem 2</li>
  <li>Custom menuitem</li>
  <li>Other menuitem</li>
  </ul>

Rendering order depends on the addon directory names.
Directory /MyDjangoProject/addons is read, sorted and then appended to INSTALLED_APPS.
To force ordering you can prepend addon directories with numbers like this:

  /MyDjangoProject/addons/01_MyOtherAddon
  /MyDjangoProject/addons/02_MyAddon

NB! For the addon directory name under templates/ has to be same as directory name under addons/ 

"""

register = template.Library()

# It would be to heavy to load template again for each node
template_cache = {}

class HookNode(Node):
    def __init__(self, nodelist, template, block_name):
        if nodelist:
            self.nodelist = nodelist
        self.template = template
        self.block_name = block_name

    def render(self, context):
        if not settings.ENABLE_ADDONS:
            return ""
        template = self.template
        retval = ""

        for addon in settings.ADDONS:
            path = "%s/%s" % (addon, template)
            if path in template_cache:                
                addon_template = template_cache[path]

            else:
                try:
                    source, origin = find_template_source(path)
                    addon_template = get_template_from_string(source, origin, template)
                    template_cache[path] = addon_template
                except TemplateDoesNotExist:
                    addon_template = None
                    template_cache[path] = None
                
            if addon_template:
                # Block is not defined, include whole file
                #print "Block not defined, including whole file..."
                if not self.block_name:
                    retval += addon_template.render(context)
                # Block is defined so look up for the block in the template
                else:
                    for node in addon_template.nodelist:
                        if isinstance(node, BlockNode) and \
                            node.name == self.block_name:
                            #print "Inserting node %s from file..." % self.block_name
                            retval += node.render(context)
        # We didn't include anything - keep parent's content
        if retval == "":
            if hasattr(self, "nodelist"):
                return self.nodelist.render(context)
        else:
            return retval
        return ""

def do_hook(parser, token):
     bits = token.split_contents()
     tag = bits[0]
     if len(bits) < 2 or len(bits) > 3:
         raise TemplateSyntaxError, "%r tag arguments are: template [block_name]" % tag
     template = bits[1]
     try:
         _,_,block_name = bits
     except:
         block_name = None

     if template[0] in ('"', "'") and template[-1] == template[0]:
         template = template[1:-1] # Strip quotes
     else:
         raise TemplateSyntaxError, "%r tag requires 'template' argument to be quoted" % tag

     #regexp = "[^A-Za-z0-9_-]" # Do not allow including /etc/passwd or something like that
     #if re.search(regexp, addon_name) == None:

     if tag == "hookblock":
         nodelist = parser.parse(('endhookblock',))
         parser.delete_first_token()
     else:
         nodelist = None
     return HookNode(nodelist, template, block_name)

register.tag('hook', do_hook)
register.tag('hookblock', do_hook)



# -*- coding: utf-8 -*-
from codecs import BOM
import os
from setuptools import setup, find_packages
from django_addons import get_version

readme_file = open(u'README')
long_description = readme_file.read()
readme_file.close()
if long_description.startswith(BOM):
    long_description = long_description.lstrip(BOM)
long_description = long_description.decode('utf-8')


setup(name='django-addons',
      version=get_version().replace(' ', '-'),
      description='Addon framework for Django',
      long_description=long_description,
      author='Lauri Võsandi',
      author_email='lauri@indifex.com',
      url='http://www.bitbucket.org/indifex/django-addons/wiki/',
      download_url='http://bitbucket.org/indifex/django-addons/get/tip.gz',
      zip_safe = False,
      packages=find_packages(),
      classifiers=['Development Status :: 3 - Alpha',
                   'Environment :: Web Environment',
                   'Framework :: Django',
                   'Intended Audience :: Developers',
                   'License :: OSI Approved :: BSD License',
                   'Operating System :: OS Independent',
                   'Programming Language :: Python',
                   'Topic :: Software Development :: Libraries :: Python Modules',
                   'Topic :: Utilities'],
)



# -*- coding: utf-8 -*-

class AddonIncompatible(BaseException):
    """
    This should be thrown when addon wants to import
    something from the base Django project but can't find it.

    This usually should indicate that addon has been installed onto
    wrong Django project
    """
    pass

class AddonError(StandardError):
    """
    Base class for all addon errors.
    """
    pass
